local function L(text) return text end  -- Localization stub

Pol_Monitor = LibStub("AceAddon-3.0"):NewAddon("PolMonitor", "AceConsole-3.0")

local options = {
	type="group",
	handler = Pol_Monitor,
	args = {					
		group = {
			type = "group",
			name = L("Button Groups"),
			desc = L("Group management of button monitors"),
			args = {
				add = {
					type = "input",
					name = L("Add new monitor group"),
					desc = L("Monitor buttons are created and categorized within groups.\n\nType the name of the group you wish to create, then expand the group created below to add new button monitors to that group.\n"),
					get = false,
					set = "OnAddGroup",
					usage = L("<Group Name>"),
					order = 1,
				},
			},
			order = 1,
		},
			
		global = {
			type = "group",
			name = L("Masque"),
			desc = L("Settings for intregration with Masque skins.  These options will be disabled if the Masque is not detected."),
			args = {
				buttonFacade = {
					type = "toggle",
					name = L("Register with Masque"),
					desc = L("If set, groups will be registered with Masque for skinning"),
					get = function(info) return Pol_Monitor:GetButtonFacadeToggle() end,
					set = function(info, v) Pol_Monitor:SetButtonFacadeToggle(v) end,
					disabled = not IsAddOnLoaded("Masque"),
					order = 2,
				},				
			},
			order = 2,			
		},
	},
}

local defaults = {
	profile = {groups={}},
	global = {spellTextures={}, itemTextures={}},
}


-- ------------------------------------------------------------------------------------------------
-- Event handlers
-- ------------------------------------------------------------------------------------------------

local inCombatState = false and (InCombatLockdown() == 1)
local inPetBattle   = false

function Pol_Monitor:GetFrameState(dtype)

	if dtype == nil or dtype == "" or dtype == "always" then return true end

	if dtype == "any" and (UnitExists("target") or inCombatState) then return true end
	
	if dtype == "combo" and (UnitCanAttack("player", "target") or inCombatState) then return true end
	
	if (dtype == "combat" or dtype == "combo") and inCombatState then return true end
				
	if dtype == "ooc" and (not inCombatState) then return true end
	
	return false	
end

function Pol_Monitor:OnInitialize()
    -- Rescue old ace2 cached icon paths if needed
	if Pol_MonitorDB and Pol_MonitorDB.account and not Pol_MonitorDB.global then
		Pol_MonitorDB.global = Pol_MonitorDB.account
		Pol_MonitorDB.account = nil
	end
	
	-- Back to normal
    self.db = LibStub("AceDB-3.0"):New("PolMonitorDB", defaults)
	self.groupObservers = {}
	self.groups = {}
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PolMonitor", options)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	-- Add dual-spec support
	local LibDualSpec = LibStub('LibDualSpec-1.0')
	LibDualSpec:EnhanceDatabase(self.db, "PolMonitor")
	LibDualSpec:EnhanceOptions(options.args.profile, self.db)

    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PolMonitor", "PolMonitor")
	self:RegisterChatCommand("polmonitor", "ChatCommand")
	self:RegisterChatCommand("pm", "ChatCommand")
	
	print ("|cff33ff99PolMonitor: |cffffffffType /pm or /polmonitor for configuration options!")
	
	-- We need to install this crazy event handler here, so we can handle Pet battle weirdness
	
	local function globalEvents(self, event, ...)
	
		if event == "UNIT_ENTERING_VEHICLE" then
		
		elseif event == "PET_BATTLE_OPENING_START" then
			
			inPetBattle = true
			
		elseif event == "PET_BATTLE_CLOSE" then

			inPetBattle = false

		elseif event == "ADDON_LOADED" then
			
		elseif event == "PLAYER_TARGET_CHANGED" then
		
		elseif event == "PLAYER_REGEN_ENABLED" then
		
			inCombatState = false
		
		elseif event == "PLAYER_REGEN_DISABLED" then
		
			inCombatState = true
			
		end
	end
	
	local eventFrame = CreateFrame("FRAME", "PolMonitorFrame")
	
	eventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	eventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
			
	eventFrame:SetScript("OnEvent", globalEvents)		
end

function Pol_Monitor:ChatCommand(input)
    if not input or input:trim() == "" then
        LibStub("AceConfigDialog-3.0"):Open("PolMonitor")
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(Pol_Monitor, "pm", "PolMonitor", input)
    end
end

function Pol_Monitor:OnEnable()
	self:OnProfileChanged()
end

function Pol_Monitor:OnProfileChanged()
	self.profileIsChanging = true
	-- Out with the old
	for groupName, group in pairs(self.groups) do
		self:_DeleteGroup(groupName)
	end
	-- In with the new
	for groupName, profile in pairs(self.db.profile.groups) do
		group = Pol_MonitorGroupFactory(groupName, profile)
		self:_AddGroup(groupName, group)
	end
	self.profileIsChanging = false
end

function Pol_Monitor:SaveFramePosition(frameName)

	frameName:StopMovingOrSizing()
	
	local f = frameName:GetName()
	local point, relTo, relPoint, x, y = frameName:GetPoint()
	
	relTo = (relTo or UIParent):GetName()

	--print("DEBUG SaveFramePosition p=" .. point .. " rt="..relTo.." rp="..relPoint.." x="..x.." y="..y)

	if f == "PetActionBarFrame" then
		Pol_Monitor.db.profile.fpetbar_P  = point
		Pol_Monitor.db.profile.fpetbar_RT = relTo
		Pol_Monitor.db.profile.fpetbar_RP = relPoint
		Pol_Monitor.db.profile.fpetbar_X  = x
		Pol_Monitor.db.profile.fpetbar_Y  = y		
	end
	
end

-- ------------------------------------------------------------------------------------------------
-- Command handlers
-- ------------------------------------------------------------------------------------------------

function Pol_Monitor:OnAddGroup(info, groupName, ...)
	if self.groups[groupName] then
		self:Print(L("Group '%s' already exists"):format(groupName))
		return
	end
	
	group = Pol_MonitorGroupFactory(groupName, "Grid", ...)
	
	self.db.profile.groups[groupName] = group:GetProfileTable()
	
	self:_AddGroup(groupName, group)
end

function Pol_Monitor:OnDeleteGroup(groupName)
	self.db.profile.groups[groupName] = nil
	self:_DeleteGroup(groupName)
end

-- ------------------------------------------------------------------------------------------------
-- Options handlers
-- ------------------------------------------------------------------------------------------------

function Pol_Monitor:GetButtonFacadeToggle()
	return (self.db.profile.buttonFacadeToggle or false)
	-- or self.db.profile.buttonFacadeToggle == nil
end

function Pol_Monitor:SetButtonFacadeToggle(v)
	self.db.profile.buttonFacadeToggle = v
end

-- ------------------------------------------------------------------------------------------------
-- API
-- ------------------------------------------------------------------------------------------------

function Pol_Monitor:IgnoreAsGlobalCooldown(duration)
	if (self.gcdEndTime or 0) < GetTime() then
		local gcdstart, gcdduration = GetSpellCooldown(self.db.profile.gcdSpell or "")
		
		self.gcdEndTime = (gcdstart or 0) + (gcdduration or 0)
	end
	
	return self.db.profile.gcdToggle and (self.gcdEndTime > GetTime() and duration ~= gcdduration) or false
end

function Pol_Monitor:GetSpellTexture(spellName)

	if self.db.global.spellTextures[spellName] == nil then
		self.db.global.spellTextures[spellName] = GetSpellTexture(spellName)
	end
	
	return self.db.global.spellTextures[spellName] or "Interface\\Icons\\INV_Misc_QuestionMark"
end

function Pol_Monitor:SetSpellTexture(spellName, texture)

	if spellName and texture then
		self.db.global.spellTextures[spellName] = texture
	end
end

function Pol_Monitor:GetItemTexture(itemName)

	if self.db.global.itemTextures[itemName] == nil then
		local _, _, _, _, _, _, _, _, _, invTexture = GetItemInfo(itemName)
		self.db.global.itemTextures[itemName] = invTexture
	end
	
	return self.db.global.itemTextures[itemName] or "Interface\\Icons\\INV_Misc_QuestionMark"
end

function Pol_Monitor:GetGroups()

	local groups = {}
	
	for k, group in pairs(self.groups) do
		table.insert(groups, group)
	end
	
	return groups
end

function Pol_Monitor:RegisterGroupObserver(key, callback)
	self.groupObservers[key] = callback
end

function Pol_Monitor:UnRegisterGroupObserver(key)
	self.groupObservers[key] = nil
end

function Pol_Monitor:NotifyGroupObservers(action, group, monitor)
	if not self.profileIsChanging or not monitor then
		for key, callback in pairs(self.groupObservers) do
			callback(action, group, monitor)
		end
	end
end

function Pol_Monitor:RepaintAllButtons()
	for k, group in pairs(self.groups) do
		group:RepaintAllButtons()
	end
end

-- ------------------------------------------------------------------------------------------------
-- Internals
-- ------------------------------------------------------------------------------------------------

function Pol_Monitor:_AddGroup(groupName, group)

	self.groups[groupName] = group
		
	options.args.group.args[groupName] = group:GetOptionsTable(10)
	
	options.args.group.args[groupName].args.deleteSeparator = {
		type = "header",
		name = "",
		order = -2,
	}
	options.args.group.args[groupName].args.delete = {
		type = "execute",
		name = L("Delete Group"),
		desc = L("Delete this group"),
		func = function() self:OnDeleteGroup(groupName) end,
		confirm = true,
		confirmText = "Delete group " .. groupName .."?",
		order = -1,
	}
end

function Pol_Monitor:_DeleteGroup(groupName)

	self.groups[groupName]:delete()
	self.groups[groupName] = nil
	
	options.args.group.args[groupName] = nil
end